@@ -52,15 +52,12 @@ INSTALLED_APPS = [
     'django.contrib.sessions',
     'django.contrib.messages',
     'django.contrib.staticfiles',
-    'server.apps.ServerConfig',
     'api.apps.ApiConfig',
-    # 'widget_tweaks',
     'rest_framework',
     'rest_framework.authtoken',
     'django_filters',
     'social_django',
     'polymorphic',
-    # 'webpack_loader',
     'corsheaders',
     'drf_yasg'
 ]
@@ -124,17 +121,6 @@ STATICFILES_DIRS = [
 
 STATICFILES_STORAGE = 'whitenoise.storage.CompressedManifestStaticFilesStorage'
 
-WEBPACK_LOADER = {
-    'DEFAULT': {
-        'CACHE': not DEBUG,
-        'BUNDLE_DIR_NAME': 'bundle/',
-        'STATS_FILE': path.join(BASE_DIR, 'server', 'static', 'webpack-stats.json'),
-        'POLL_INTERVAL': 0.1,
-        'TIMEOUT': None,
-        'IGNORE': [r'.*\.hot-update.js', r'.+\.map']
-    }
-}
-
 WSGI_APPLICATION = 'app.wsgi.application'
 
 AUTHENTICATION_BACKENDS = [
@@ -280,7 +266,8 @@ LOGOUT_REDIRECT_URL = '/'
 
 # dynamic import to avoid installing psycopg2 on pip installation.
 name = 'django_heroku'
-if (spec := importlib.util.find_spec(name)) is not None:
+spec = importlib.util.find_spec(name)
+if spec is not None:
     module = importlib.util.module_from_spec(spec)
     sys.modules[name] = module
     spec.loader.exec_module(module)
